@@ -35,6 +35,7 @@
 
 class LayoutItem_GroupBy;
 class LayoutItem_Summary;
+class LayoutItem_VerticalGroup;
 
 /** A base class that is a Bakery View with some database functionality.
 */
@@ -83,17 +84,17 @@ public:
   typedef std::vector< sharedptr<LayoutItem_Field> > type_vecLayoutFields;
 
 protected:
-  sharedptr<LayoutItem_Field> offer_field_list(const Glib::ustring& table_name);
-  sharedptr<LayoutItem_Field> offer_field_list(const sharedptr<LayoutItem_Field>& start_field, const Glib::ustring& table_name);
+  sharedptr<LayoutItem_Field> offer_field_list(const Glib::ustring& table_name, Gtk::Window* transient_for = 0);
+  sharedptr<LayoutItem_Field> offer_field_list(const sharedptr<const LayoutItem_Field>& start_field, const Glib::ustring& table_name, Gtk::Window* transient_for = 0);
+  sharedptr<LayoutItem_Field> offer_field_formatting(const sharedptr<const LayoutItem_Field>& start_field, const Glib::ustring& table_name, Gtk::Window* transient_for = 0);
+  sharedptr<LayoutItem_Text> offer_textobject(const sharedptr<LayoutItem_Text>& start_textobject, Gtk::Window* transient_for = 0);
+  sharedptr<LayoutItem_Notebook> offer_notebook(const sharedptr<LayoutItem_Notebook>& start_notebook, Gtk::Window* transient_for = 0);
 
   ///@result Whether the user would like to find again.
   static bool show_warning_no_records_found(Gtk::Window& transient_for);
 
   void fill_full_field_details(const Glib::ustring& parent_table_name, sharedptr<LayoutItem_Field>& layout_item);
 
-  ///Get the table name. It's either the current table or the relationship's to_table:
-  Glib::ustring get_layout_item_table_name(const sharedptr<const LayoutItem_Field>& layout_item, const Glib::ustring& table_name);
-
   typedef std::vector<Glib::ustring> type_vecStrings;
   type_vecStrings get_table_names(bool ignore_system_tables = false) const;
 
@@ -126,22 +127,33 @@ protected:
     : m_key_value(key_value)
     {
       m_field = layout_item->get_full_field_details();
+      m_table_name = layout_item->get_table_used(parent_table_name);
 
+      //The key:
       if(layout_item->get_has_relationship_name())
       {
         //The field is in a related table.
         sharedptr<const Relationship> rel = layout_item->get_relationship();
         if(rel)
         {
-          m_table_name = rel->get_to_table();
-
-          //Actually a foreign key:
-          m_key = document.get_field(m_table_name, rel->get_to_field());
+          if(layout_item->get_has_related_relationship_name()) //For doubly-related fields
+          {
+            sharedptr<const Relationship> rel = layout_item->get_related_relationship();
+            if(rel)
+            {
+              //Actually a foreign key in a doubly-related table:
+              m_key = document.get_field(m_table_name, rel->get_to_field());
+            }
+          }
+          else
+          {
+            //Actually a foreign key:
+            m_key = document.get_field(m_table_name, rel->get_to_field());
+          }
         }
       }
       else
       {
-        m_table_name = parent_table_name;
         m_key = parent_key;
       }
     }
@@ -223,11 +235,19 @@ protected:
   void set_database_preferences(const SystemPrefs& prefs);
 
   void report_build(const Glib::ustring& table_name, const sharedptr<const Report>& report, const Glib::ustring& where_clause);
+
   void report_build_groupby(const Glib::ustring& table_name, xmlpp::Element& parent_node, const sharedptr<LayoutItem_GroupBy>& group_by, const Glib::ustring& where_clause_parent);
+  void report_build_groupby_children(const Glib::ustring& table_name, xmlpp::Element& nodeGroupBy, const sharedptr<LayoutItem_GroupBy>& group_by, const Glib::ustring& where_clause);
   void report_build_summary(const Glib::ustring& table_name, xmlpp::Element& parent_node, const sharedptr<LayoutItem_Summary>& summary, const Glib::ustring& where_clause_parent);
 
-  ///@show_null_records means show a summary of zero values when there are no records to actually summarise.
-  void report_build_records(const Glib::ustring& table_name, xmlpp::Element& parent_node, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const Glib::ustring& sort_clause = Glib::ustring(), bool one_record_only = false);
+  typedef std::vector< sharedptr<LayoutItem> > type_vecLayoutItems;
+
+  void report_build_records(const Glib::ustring& table_name, xmlpp::Element& parent_node, const type_vecLayoutItems& items, const Glib::ustring& where_clause, const Glib::ustring& sort_clause = Glib::ustring(), bool one_record_only = false);
+  void report_build_records_get_fields(const Glib::ustring& table_name, const sharedptr<LayoutGroup>& group, type_vecLayoutFields& items);
+  void report_build_records_field(const Glib::ustring& table_name, xmlpp::Element& nodeParent, const sharedptr<const LayoutItem_Field>& field, const Glib::RefPtr<Gnome::Gda::DataModel>& datamodel, guint row, guint& colField, bool vertical = false);
+  void report_build_records_text(const Glib::ustring& table_name, xmlpp::Element& nodeParent, const sharedptr<const LayoutItem_Text>& textobject);
+  void report_build_records_vertical_group(const Glib::ustring& table_name, xmlpp::Element& vertical_group_node, const sharedptr<LayoutItem_VerticalGroup>& group, const Glib::RefPtr<Gnome::Gda::DataModel>& datamodel, guint row, guint& field_index);
+
 
   Gnome::Gda::Value auto_increment_insert_first_if_necessary(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
 
