@@ -17,7 +17,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
- 
+
 #ifndef GLOM_DATASTRUCTURE_LAYOUTGROUP_H
 #define GLOM_DATASTRUCTURE_LAYOUTGROUP_H
 
@@ -62,8 +62,13 @@ public:
 
   void remove_all_items();
 
+  double get_border_width() const;
+  void set_border_width(double border_width);
+
   //virtual void debug(guint level = 0) const;
 
+  guint get_items_count() const;
+
   guint m_columns_count;
 
   typedef std::map<int, sharedptr<LayoutItem> > type_map_items;
@@ -81,6 +86,8 @@ public:
 protected:
 
   void remove_item(guint sequence);
+
+  double m_border_width; //For use on reports.
 };
 
 #endif //GLOM_DATASTRUCTURE_LAYOUTGROUP_H
