@@ -22,9 +22,12 @@
 #include "data_structure/layout/report_parts/layoutitem_groupby.h"
 #include "data_structure/layout/report_parts/layoutitem_summary.h"
 #include "data_structure/layout/report_parts/layoutitem_fieldsummary.h"
+#include "data_structure/layout/report_parts/layoutitem_verticalgroup.h"
 #include "data_structure/layout/layoutitem_field.h"
+#include "data_structure/layout/layoutitem_text.h"
 #include "mode_data/dialog_choose_field.h"
 #include "layout_item_dialogs/dialog_field_layout.h"
+#include "mode_design/dialog_textobject.h"
 #include "layout_item_dialogs/dialog_group_by.h"
 #include "layout_item_dialogs/dialog_field_summary.h"
 #include "mode_data/dialog_choose_relationship.h"
@@ -42,6 +45,7 @@ Dialog_Layout_Report::Dialog_Layout_Report(BaseObjectType* cobject, const Glib::
   m_button_add(0),
   m_button_delete(0),
   m_button_edit(0),
+  m_button_formatting(0),
   m_label_table_name(0),
   m_entry_name(0),
   m_entry_title(0)
@@ -60,13 +64,21 @@ Dialog_Layout_Report::Dialog_Layout_Report(BaseObjectType* cobject, const Glib::
 
     Gtk::TreeModel::iterator iter = m_model_available_parts->append();
     (*iter)[m_columns_available_parts.m_col_item] = sharedptr<LayoutItem>(static_cast<LayoutItem*>(new LayoutItem_GroupBy()));
-    iter = m_model_available_parts->append(iter->children()); //Place Field under GroupBy to indicate that that's where it belongs in the actual layout.
-    (*iter)[m_columns_available_parts.m_col_item] = sharedptr<LayoutItem>(static_cast<LayoutItem*>(new LayoutItem_Field()));
+
+    Gtk::TreeModel::iterator iterField = m_model_available_parts->append(iter->children()); //Place Field under GroupBy to indicate that that's where it belongs in the actual layout.
+    (*iterField)[m_columns_available_parts.m_col_item] = sharedptr<LayoutItem>(static_cast<LayoutItem*>(new LayoutItem_Field()));
+
+    Gtk::TreeModel::iterator iterText = m_model_available_parts->append(iter->children());
+    (*iterText)[m_columns_available_parts.m_col_item] = sharedptr<LayoutItem>(static_cast<LayoutItem*>(new LayoutItem_Text()));
+
+    Gtk::TreeModel::iterator iterVerticalGroup = m_model_available_parts->append(iter->children());
+    (*iterVerticalGroup)[m_columns_available_parts.m_col_item] = sharedptr<LayoutItem>(static_cast<LayoutItem*>(new LayoutItem_VerticalGroup()));
+
     iter = m_model_available_parts->append();
     (*iter)[m_columns_available_parts.m_col_item] = sharedptr<LayoutItem>(static_cast<LayoutItem*>(new LayoutItem_Summary()));
     iter = m_model_available_parts->append(iter->children());
     (*iter)[m_columns_available_parts.m_col_item] = sharedptr<LayoutItem>(static_cast<LayoutItem*>(new LayoutItem_FieldSummary()));
-    
+
     m_treeview_available_parts->set_model(m_model_available_parts);
     m_treeview_available_parts->expand_all();
 
@@ -150,6 +162,9 @@ Dialog_Layout_Report::Dialog_Layout_Report(BaseObjectType* cobject, const Glib::
   refGlade->get_widget("button_edit", m_button_edit);
   m_button_edit->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Report::on_button_edit) );
 
+  refGlade->get_widget("button_formatting", m_button_formatting);
+  m_button_formatting->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Report::on_button_formatting) );
+
   show_all_children();
 }
 
@@ -206,20 +221,20 @@ void Dialog_Layout_Report::fill_group_children(const sharedptr<LayoutGroup>& gro
 
 void Dialog_Layout_Report::add_group(const Gtk::TreeModel::iterator& parent, const sharedptr<const LayoutGroup>& group)
 {
-  Gtk::TreeModel::iterator iterNewGroup;
+  Gtk::TreeModel::iterator iterNewItem;
   if(!parent)
   {
     //Add it at the top-level, because nothing was selected:
-    iterNewGroup = m_model_parts->append();
+    iterNewItem = m_model_parts->append();
   }
   else
   {
-    iterNewGroup = m_model_parts->append(parent->children());
+    iterNewItem = m_model_parts->append(parent->children());
   }
 
-  if(iterNewGroup)
+  if(iterNewItem)
   {
-    Gtk::TreeModel::Row row = *iterNewGroup;
+    Gtk::TreeModel::Row row = *iterNewItem;
 
     row[m_columns_parts.m_col_item] = sharedptr<LayoutItem>(static_cast<LayoutItem*>(group->clone()));
 
@@ -230,13 +245,19 @@ void Dialog_Layout_Report::add_group(const Gtk::TreeModel::iterator& parent, con
 
       sharedptr<const LayoutGroup> pGroup = sharedptr<const LayoutGroup>::cast_dynamic(pItem);
       if(pGroup)
-        add_group(iterNewGroup /*parent of the new group */, pGroup); //recurse
+        add_group(iterNewItem /*parent of the new group */, pGroup); //recurse
       else
       {
-        Gtk::TreeModel::iterator iterChildRow = m_model_parts->append(iterNewGroup->children());
+        Gtk::TreeModel::iterator iterChildRow = m_model_parts->append(iterNewItem->children());
 
         Gtk::TreeModel::Row row = *iterChildRow;
-        row[m_columns_parts.m_col_item] = sharedptr<LayoutItem>(pItem->clone());
+
+        sharedptr<LayoutItem> pCloned = sharedptr<LayoutItem>(pItem->clone());
+        row[m_columns_parts.m_col_item] = pCloned;
+
+        //sharedptr<LayoutItem_Field> pField = sharedptr<LayoutItem_Field>::cast_dynamic(pCloned);
+        //if(pField)
+        //  fill_full_field_details(m_table_name, pField);
       }
     }
 
@@ -278,9 +299,16 @@ void Dialog_Layout_Report::set_report(const Glib::ustring& table_name, const sha
     //guint group_sequence = 1; //0 means no sequence
     for(LayoutGroup::type_map_items::const_iterator iter = report->m_layout_group->m_map_items.begin(); iter != report->m_layout_group->m_map_items.end(); ++iter)
     {
-      sharedptr<const LayoutGroup> group = sharedptr<const LayoutGroup>::cast_dynamic(iter->second);
+      sharedptr<const LayoutItem> item = iter->second;
+      sharedptr<const LayoutGroup> group = sharedptr<const LayoutGroup>::cast_dynamic(item);
       if(group)
         add_group(Gtk::TreeModel::iterator() /* null == top-level */, group);
+      else
+      {
+        Gtk::TreeModel::iterator iter = m_model_parts->append();
+        Gtk::TreeModel::Row row = *iter;
+        row[m_columns_parts.m_col_item] = glom_sharedptr_clone(item);
+      }
     }
 
     //treeview_fill_sequences(m_model_parts, m_model_parts->m_columns.m_col_sequence); //The document should have checked this already, but it does not hurt to check again.
@@ -294,6 +322,9 @@ void Dialog_Layout_Report::set_report(const Glib::ustring& table_name, const sha
 
 bool Dialog_Layout_Report::may_be_child_of(const sharedptr<const LayoutItem>& parent, const sharedptr<const LayoutItem>& suggested_child)
 {
+  if(!parent)
+    return true; //Anything may be at the top-level.
+
   if(!(sharedptr<const LayoutGroup>::cast_dynamic(parent)))
     return false; //Only LayoutGroup (and derived types) may have children.
 
@@ -365,6 +396,14 @@ void Dialog_Layout_Report::enable_buttons()
 
       m_button_delete->set_sensitive(true);
 
+      //The [Formatting] button:
+      bool enable_formatting = false;
+      sharedptr<LayoutItem_Field> item_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item_parent);
+      if(item_field)
+        enable_formatting = true; 
+
+      m_button_formatting->set_sensitive(enable_formatting);
+
       //m_button_formatting->set_sensitive( (*iter)[m_columns_parts->m_columns.m_col_type] == TreeStore_Layout::TYPE_FIELD);
     }
     else
@@ -373,13 +412,29 @@ void Dialog_Layout_Report::enable_buttons()
       m_button_down->set_sensitive(false);
       m_button_up->set_sensitive(false);
       m_button_delete->set_sensitive(false);
+      m_button_formatting->set_sensitive(false);
+    }
+
+     //Not all parts may be children of all other parts.
+    if(layout_item_available && layout_item_parent)
+    {
+      const bool may_be_child_of_parent = may_be_child_of(layout_item_parent, layout_item_available);
+      enable_add = may_be_child_of_parent;
+
+      if(!may_be_child_of_parent)
+      {
+        //Maybe it can be a sibling of the parent instead (and that's what would happen if Add was clicked).
+        sharedptr<LayoutItem> layout_item_parent_of_parent;
+
+        Gtk::TreeModel::iterator iterParent = iter->parent();
+        if(iterParent)
+          layout_item_parent_of_parent = (*iterParent)[m_columns_parts.m_col_item];
+
+        enable_add = may_be_child_of(layout_item_parent_of_parent, layout_item_available);
+      }
     }
   }
 
-  //Not all parts may be children of all other parts.
-  if(layout_item_available && layout_item_parent)
-    enable_add = may_be_child_of(layout_item_parent, layout_item_available);
-
   m_button_add->set_sensitive(enable_add);
 }
 
@@ -526,41 +581,6 @@ sharedptr<Relationship> Dialog_Layout_Report::offer_relationship_list()
   return result;
 }
 
-sharedptr<LayoutItem_Field> Dialog_Layout_Report::offer_field_layout(const sharedptr<LayoutItem_Field>& start_field)
-{
-  sharedptr<LayoutItem_Field> result;
-
-  try
-  {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_layout_field_properties");
-
-    Dialog_FieldLayout* dialog = 0;
-    refXml->get_widget_derived("dialog_layout_field_properties", dialog);
-
-    if(dialog)
-    {
-      add_view(dialog); //Give it access to the document.
-      dialog->set_field(start_field, m_table_name);
-      dialog->set_transient_for(*this);
-      int response = dialog->run();
-      if(response == Gtk::RESPONSE_OK)
-      {
-        //Get the chosen field:
-        result = dialog->get_field_chosen();
-      }
-
-      remove_view(dialog);
-      delete dialog;
-    }
-  }
-  catch(const Gnome::Glade::XmlError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
-  }
-
-  return result;
-}
-
 Gtk::TreeModel::iterator Dialog_Layout_Report::get_selected_group_parent() const
 {
   //Get the selected group, or a suitable parent group, or the first group:
@@ -606,6 +626,33 @@ Gtk::TreeModel::iterator Dialog_Layout_Report::get_selected_available() const
   return iter;
 }
 
+
+void Dialog_Layout_Report::on_button_formatting()
+{
+  //Get the selected item:
+  Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_parts->get_selection();
+  if(refTreeSelection)
+  {
+    Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
+    if(iter)
+    {
+      Gtk::TreeModel::Row row = *iter;
+      sharedptr<LayoutItem> item = row[m_columns_parts.m_col_item];
+
+      sharedptr<LayoutItem_Field> field = sharedptr<LayoutItem_Field>::cast_dynamic(item);
+      if(field)
+      {
+        sharedptr<LayoutItem_Field> field_chosen = offer_field_formatting(field, m_table_name, this);
+        if(field_chosen)
+        {
+          *field = *field_chosen;
+          m_model_parts->row_changed(Gtk::TreePath(iter), iter);
+        }
+      }
+    }
+  }
+}
+
 void Dialog_Layout_Report::on_button_edit()
 {
   //Get the selected item:
@@ -635,7 +682,7 @@ void Dialog_Layout_Report::on_button_edit()
             dialog->set_item(fieldsummary, m_table_name);
             dialog->set_transient_for(*this);
 
-            int response = dialog->run();
+            const int response = dialog->run();
             dialog->hide();
 
             if(response == Gtk::RESPONSE_OK)
@@ -647,6 +694,7 @@ void Dialog_Layout_Report::on_button_edit()
                 *fieldsummary = *chosenitem; //TODO_Performance.
 
                 m_model_parts->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+                m_modified = true;
               }
             }
 
@@ -664,52 +712,68 @@ void Dialog_Layout_Report::on_button_edit()
         sharedptr<LayoutItem_Field> field = sharedptr<LayoutItem_Field>::cast_dynamic(item);
         if(field)
         {
-          sharedptr<LayoutItem_Field> chosenitem = offer_field_list(field, m_table_name);
+          sharedptr<LayoutItem_Field> chosenitem = offer_field_list(field, m_table_name, this);
           if(chosenitem)
           {
             *field = *chosenitem; //TODO_Performance.
             m_model_parts->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+            m_modified = true;
           }
         }
         else
         {
-          sharedptr<LayoutItem_GroupBy> group_by = sharedptr<LayoutItem_GroupBy>::cast_dynamic(item);
-          if(group_by)
+          sharedptr<LayoutItem_Text> layout_item_text = sharedptr<LayoutItem_Text>::cast_dynamic(item);
+          if(layout_item_text)
           {
-            try
+            sharedptr<LayoutItem_Text> chosen = offer_textobject(layout_item_text);
+            if(chosen)
             {
-              Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_group_by");
-
-              Dialog_GroupBy* dialog = 0;
-              refXml->get_widget_derived("dialog_group_by", dialog);
-
-              if(dialog)
-              {
-                add_view(dialog);
-                dialog->set_item(group_by, m_table_name);
-                dialog->set_transient_for(*this);
-
-                const int response = dialog->run();
-                dialog->hide();
-
-                if(response == Gtk::RESPONSE_OK)
-                {
-                  //Get the chosen relationship:
-                  sharedptr<LayoutItem_GroupBy> chosenitem = dialog->get_item();
-                  if(chosenitem)
-                  {
-                    *group_by = *chosenitem;
-                    m_model_parts->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
-                  }
-                }
-
-                remove_view(dialog);
-                delete dialog;
-              }
+              *layout_item_text = *chosen;
+              m_model_parts->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+              m_modified = true;
             }
-            catch(const Gnome::Glade::XmlError& ex)
+          }
+          else
+          {
+            sharedptr<LayoutItem_GroupBy> group_by = sharedptr<LayoutItem_GroupBy>::cast_dynamic(item);
+            if(group_by)
             {
-              std::cerr << ex.what() << std::endl;
+              try
+              {
+                Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_group_by");
+
+                Dialog_GroupBy* dialog = 0;
+                refXml->get_widget_derived("dialog_group_by", dialog);
+
+                if(dialog)
+                {
+                  add_view(dialog);
+                  dialog->set_item(group_by, m_table_name);
+                  dialog->set_transient_for(*this);
+
+                  const int response = dialog->run();
+                  dialog->hide();
+
+                  if(response == Gtk::RESPONSE_OK)
+                  {
+                    //Get the chosen relationship:
+                    sharedptr<LayoutItem_GroupBy> chosenitem = dialog->get_item();
+                    if(chosenitem)
+                    {
+                      *group_by = *chosenitem;
+                      m_model_parts->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+                      m_modified = true;
+                    }
+                  }
+
+                  remove_view(dialog);
+                  delete dialog;
+                }
+              }
+              catch(const Gnome::Glade::XmlError& ex)
+              {
+                std::cerr << ex.what() << std::endl;
+              }
             }
           }
         }
@@ -787,7 +851,7 @@ void Dialog_Layout_Report::on_cell_data_part(Gtk::CellRenderer* renderer, const
     {
       Gtk::TreeModel::Row row = *iter;
       sharedptr<LayoutItem> pItem = row[m_columns_parts.m_col_item];
-      Glib::ustring part = pItem->get_part_type_name();
+      const Glib::ustring part = pItem->get_part_type_name();
 
       renderer_text->property_text() = part;
       renderer_text->property_editable() = false; //Part names can never be edited.
@@ -807,35 +871,7 @@ void Dialog_Layout_Report::on_cell_data_details(Gtk::CellRenderer* renderer, con
 
       Gtk::TreeModel::Row row = *iter;
       sharedptr<LayoutItem> pItem = row[m_columns_parts.m_col_item];
-
-      sharedptr<LayoutItem_GroupBy> pGroup = sharedptr<LayoutItem_GroupBy>::cast_dynamic(pItem);
-      if(pGroup)
-      {
-        //TODO: Internationalize this properly:
-        if(pGroup->get_has_field_group_by())
-          text = pGroup->get_field_group_by()->get_layout_display_name();
-
-        if(pGroup->get_has_field_sort_by())
-          text += "(sort by: " + pGroup->get_field_sort_by()->get_layout_display_name() + ")";
-      }
-      else
-      {
-        /*
-        const LayoutItem_FieldSummary* pFieldSummary = dynamic_cast<const LayoutItem_FieldSummary*>(pItem);
-        if(pFieldSummary)
-        {
-           text = pFieldSummary.LayoutItem_Field::get_layout_display_name_field();
-        }
-        else
-        {
-        */
-          sharedptr<LayoutItem_Field> pField = sharedptr<LayoutItem_Field>::cast_dynamic(pItem);
-          if(pField)
-            text = pField->get_layout_display_name();
-        //}
-      }
-
-      renderer_text->property_text() = text;
+      renderer_text->property_text() = pItem->get_layout_display_name();
       renderer_text->property_editable() = false;
     }
   }
@@ -874,15 +910,32 @@ sharedptr<Report> Dialog_Layout_Report::get_report()
 
   m_report->m_layout_group->remove_all_items();
 
-  guint group_sequence = 0;
+  guint item_sequence = 0;
   m_report->m_layout_group->remove_all_items();
+
   for(Gtk::TreeModel::iterator iter = m_model_parts->children().begin(); iter != m_model_parts->children().end(); ++iter)
   {
+    //Recurse into a group if necessary:
     sharedptr<LayoutGroup> group = fill_group(iter);
-    group->m_sequence = group_sequence;
+    if(group)
+    {
+      //Add the group:
+      group->m_sequence = item_sequence;
 
-    m_report->m_layout_group->m_map_items[group_sequence] = group;
-    ++group_sequence;
+      m_report->m_layout_group->m_map_items[item_sequence] = group;
+      ++item_sequence;
+    }
+    else
+    {
+      sharedptr<LayoutItem> item = (*iter)[m_columns_parts.m_col_item];
+      if(item)
+      {
+        item->m_sequence = item_sequence;
+
+        m_report->m_layout_group->m_map_items[item_sequence] = item;
+        ++item_sequence;
+      }
+    }
   }
 
   return m_report;
