@@ -458,9 +458,16 @@ void Frame_Glom::on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction
       bool test = get_user_is_in_group(connection_pool->get_user(), GLOM_STANDARD_GROUP_NAME_DEVELOPER);
       if(test)
       {
+        std::cout << "DEBUG: User=" << connection_pool->get_user() << " _is_ in the developer group on the server." << std::endl;
         //Avoid double signals:
         //if(document->get_userlevel() != AppState::USERLEVEL_DEVELOPER)
         test = document->set_userlevel(AppState::USERLEVEL_DEVELOPER);
+        if(!test)
+          std::cout << "  DEBUG: But document->set_userlevel(AppState::USERLEVEL_DEVELOPER) failed." << std::endl;
+      }
+      else
+      {
+        std::cout << "DEBUG: User=" << connection_pool->get_user() << " is _not_ in the developer group on the server." << std::endl;
       }
 
       //If this was not possible then revert the menu:
@@ -1335,11 +1342,11 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib:
       Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
       if(connection)
       {
-        bool result = connection->create_database(database_name);
-        if(result)
-        {
-          std::cout << "Frame_Glom::create_database(): Creation succeeded: database_name=" << database_name << std::endl;
-        }
+        connection->create_database(database_name);
+        //if(result)
+        //{
+        //  std::cout << "Frame_Glom::create_database(): Creation succeeded: database_name=" << database_name << std::endl;
+        //}
       }
     }
   }
@@ -1367,19 +1374,19 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib:
     //If we got this far then the user must really have developer privileges already:
     add_standard_groups();
 
-    std::cout << "Frame_Glom::create_database(): Creation of standard tables and groups finished." << std::endl;
+    //std::cout << "Frame_Glom::create_database(): Creation of standard tables and groups finished." << std::endl;
 
     //Set the title based on the title in the example document, or the user-supplied title when creating new documents:
     SystemPrefs prefs = get_database_preferences();
     if(prefs.m_name.empty())
     {
-      std::cout << "Frame_Glom::create_database(): Setting title in the database." << std::endl;
+      //std::cout << "Frame_Glom::create_database(): Setting title in the database." << std::endl;
       prefs.m_name = title;
       set_database_preferences(prefs);
     }
     else
     {
-      std::cout << "Frame_Glom::create_database(): database has title: " << prefs.m_name << std::endl;
+      //std::cout << "Frame_Glom::create_database(): database has title: " << prefs.m_name << std::endl;
     }
 
     return true;
