@@ -26,6 +26,7 @@
 #include "../document/document_glom.h"
 #include "../box_db.h"
 #include "../utility_widgets/combo_textglade.h"
+#include "../combobox_relationship.h"
 #include "../utility_widgets/comboentry_currency.h"
 
 class Dialog_ChooseField : public Gtk::Dialog
@@ -39,19 +40,20 @@ public:
    * @param table_name The table name.
    * @param field The starting field information.
    */
-  virtual void set_document(Document_Glom* document, const Glib::ustring& table_name, const LayoutItem_Field& field);
+  virtual void set_document(Document_Glom* document, const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& field);
   virtual void set_document(Document_Glom* document, const Glib::ustring& table_name);
 
 
-  //void select_item(const Field& field);
+  //void select_item(const sharedptr<const Field>& field);
 
-  bool get_field_chosen(LayoutItem_Field& field) const;
+  sharedptr<LayoutItem_Field> get_field_chosen() const;
 
 protected:
 
   virtual void on_row_activated(const Gtk::TreePath& path, Gtk::TreeViewColumn* view_column);
   virtual void on_treeview_selection_changed();
   virtual void on_combo_relationship_changed();
+  virtual void on_checkbutton_related_relationships_toggled();
 
   //Tree model columns:
   class ModelColumns_Fields : public Gtk::TreeModel::ColumnRecord
@@ -63,23 +65,17 @@ protected:
 
     Gtk::TreeModelColumn<Glib::ustring> m_col_name;
     Gtk::TreeModelColumn<Glib::ustring> m_col_title;
-    Gtk::TreeModelColumn<Field> m_col_field;
+    Gtk::TreeModelColumn< sharedptr<Field> > m_col_field;
   };
 
   ModelColumns_Fields m_ColumnsFields;
 
-  Combo_TextGlade* m_combo_relationship;
+  ComboBox_Relationship* m_combo_relationship;
   Gtk::Button* m_button_select;
-  Gtk::CheckButton* m_checkbutton_editable;
+  Gtk::CheckButton* m_checkbutton_show_related_relationships;
   Gtk::TreeView* m_treeview;
   Glib::RefPtr<Gtk::ListStore> m_model;
 
-  Gtk::Frame* m_frame_numeric_format;
-  Gtk::CheckButton* m_checkbox_format_use_thousands;
-  Gtk::CheckButton* m_checkbox_format_use_decimal_places;
-  Gtk::Entry* m_entry_format_decimal_places;
-  ComboEntry_Currency* m_entry_currency_symbol;
-
   Glib::ustring m_table_name;
 
   Document_Glom* m_document;
