@@ -19,10 +19,11 @@
  */
 
 #include "dialog_layout_details.h"
-#include "dialog_choose_field.h"
-#include "../layout_item_dialogs/dialog_field_layout.h"
 #include "dialog_choose_relationship.h"
 #include "../mode_design/dialog_buttonscript.h"
+#include "../mode_design/dialog_textobject.h"
+#include "../layout_item_dialogs/dialog_notebook.h"
+#include "../frame_glom.h" //For show_ok_dialog()
 //#include <libgnome/gnome-i18n.h>
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include <glibmm/i18n.h>
@@ -36,8 +37,10 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   m_button_field_down(0),
   m_button_field_add(0),
   m_button_field_add_group(0),
+  m_button_add_notebook(0),
   m_button_add_related(0),
   m_button_add_button(0),
+  m_button_add_text(0),
   m_button_field_delete(0),
   m_button_field_formatting(0),
   m_button_edit(0),
@@ -122,12 +125,18 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   refGlade->get_widget("button_field_add_group", m_button_field_add_group);
   m_button_field_add_group->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_field_add_group) );
 
+  refGlade->get_widget("button_add_notebook", m_button_add_notebook);
+  m_button_add_notebook->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_add_notebook) );
+
   refGlade->get_widget("button_add_related", m_button_add_related);
   m_button_add_related->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_add_related) );
 
   refGlade->get_widget("button_add_button", m_button_add_button);
   m_button_add_button->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_add_button) );
 
+  refGlade->get_widget("button_add_text", m_button_add_text);
+  m_button_add_text->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_add_text) );
+
   refGlade->get_widget("button_edit", m_button_edit);
   m_button_edit->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_edit) );
 
@@ -264,6 +273,17 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
               Gtk::TreeModel::Row row = *iterButton;
               row[m_model_items->m_columns.m_col_layout_item] = glom_sharedptr_clone(button);
             }
+            else
+            {
+              sharedptr<const LayoutItem_Text> textobject = sharedptr<const LayoutItem_Text>::cast_dynamic(item);
+              if(textobject) //If it is a text object
+              {
+                //Add the textobject to the treeview:
+                Gtk::TreeModel::iterator iterTextObject = m_model_items->append(iterNewGroup->children());
+                Gtk::TreeModel::Row row = *iterTextObject;
+                row[m_model_items->m_columns.m_col_layout_item] = glom_sharedptr_clone(textobject);
+              }
+            }
           }
         }
       }
@@ -486,7 +506,7 @@ void Dialog_Layout_Details::on_button_field_down()
 
 void Dialog_Layout_Details::on_button_field_add()
 {
-  sharedptr<LayoutItem_Field> layout_item = offer_field_list();
+  sharedptr<LayoutItem_Field> layout_item = offer_field_list(m_table_name, this);
   if(layout_item)
   {
     //Add the field details to the layout treeview:
@@ -545,6 +565,42 @@ sharedptr<LayoutItem_Button> Dialog_Layout_Details::offer_button_script_edit(con
   return result;
 }
 
+/*
+sharedptr<LayoutItem_Text> Dialog_Layout_Details::offer_textobject_edit(const sharedptr<const LayoutItem_Text>& textobject)
+{
+  sharedptr<LayoutItem_Text> result;
+
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_textobject");
+
+    Dialog_TextObject* dialog = 0;
+    refXml->get_widget_derived("window_textobject", dialog);
+
+    if(dialog)
+    {
+      dialog->set_textobject(textobject, m_table_name);
+      dialog->set_transient_for(*this);
+      const int response = dialog->run();
+      dialog->hide();
+      if(response == Gtk::RESPONSE_OK)
+      {
+        //Get the chosen relationship:
+        result = dialog->get_textobject();
+      }
+
+      delete dialog;
+    }
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+
+  return result;
+}
+*/
+
 sharedptr<Relationship> Dialog_Layout_Details::offer_relationship_list()
 {
   sharedptr<Relationship> result;
@@ -579,79 +635,6 @@ sharedptr<Relationship> Dialog_Layout_Details::offer_relationship_list()
   return result;
 }
 
-sharedptr<LayoutItem_Field> Dialog_Layout_Details::offer_field_list()
-{
-  return offer_field_list(sharedptr<LayoutItem_Field>());
-}
-
-sharedptr<LayoutItem_Field> Dialog_Layout_Details::offer_field_list(const sharedptr<const LayoutItem_Field>& start_field)
-{
-  sharedptr<LayoutItem_Field> result;
-
-  try
-  {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_field");
-
-    Dialog_ChooseField* dialog = 0;
-    refXml->get_widget_derived("dialog_choose_field", dialog);
-
-    if(dialog)
-    {
-      dialog->set_document(get_document(), m_table_name, start_field);
-      dialog->set_transient_for(*this);
-      int response = dialog->run();
-      if(response == Gtk::RESPONSE_OK)
-      {
-        //Get the chosen field:
-        result = dialog->get_field_chosen();
-      }
-
-      delete dialog;
-    }
-  }
-  catch(const Gnome::Glade::XmlError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
-  }
-
-  return result;
-}
-
-sharedptr<LayoutItem_Field> Dialog_Layout_Details::offer_field_layout(const sharedptr<const LayoutItem_Field>& start_field)
-{
-  sharedptr<LayoutItem_Field> result;
-
-  try
-  {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_layout_field_properties");
-
-    Dialog_FieldLayout* dialog = 0;
-    refXml->get_widget_derived("dialog_layout_field_properties", dialog);
-
-    if(dialog)
-    {
-      add_view(dialog); //Give it access to the document.
-      dialog->set_field(start_field, m_table_name);
-      dialog->set_transient_for(*this);
-      int response = dialog->run();
-      if(response == Gtk::RESPONSE_OK)
-      {
-        //Get the chosen field:
-        result = dialog->get_field_chosen();
-      }
-
-      remove_view(dialog);
-      delete dialog;
-    }
-  }
-  catch(const Gnome::Glade::XmlError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
-  }
-
-  return result;
-}
-
 Gtk::TreeModel::iterator Dialog_Layout_Details::append_appropriate_row()
 {
   Gtk::TreeModel::iterator result;
@@ -707,6 +690,57 @@ void Dialog_Layout_Details::on_button_add_button()
   enable_buttons();
 }
 
+void Dialog_Layout_Details::on_button_add_text()
+{
+  Gtk::TreeModel::iterator iter = append_appropriate_row();
+  if(iter)
+  {
+    Gtk::TreeModel::Row row = *iter;
+
+    //Add a new button:
+    sharedptr<LayoutItem_Text> textobject = sharedptr<LayoutItem_Text>::create();
+    textobject->set_title(_("Text Title")); //Give the button a default title, so it is big enough, and so people see that they should change it.
+    row[m_model_items->m_columns.m_col_layout_item] = textobject;
+
+    //Scroll to, and select, the new row:
+    Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
+    if(refTreeSelection)
+      refTreeSelection->select(iter);
+
+    m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
+
+    m_modified = true;
+  }
+
+  enable_buttons();
+}
+
+void Dialog_Layout_Details::on_button_add_notebook()
+{
+  Gtk::TreeModel::iterator parent = get_selected_group_parent();
+
+  Gtk::TreeModel::iterator iter = append_appropriate_row();
+  if(iter)
+  {
+    Gtk::TreeModel::Row row = *iter;
+
+    sharedptr<LayoutItem_Notebook> notebook = sharedptr<LayoutItem_Notebook>::create();
+    notebook->set_name(_("notebook"));
+    row[m_model_items->m_columns.m_col_layout_item] = notebook;
+
+    //Scroll to, and select, the new row:
+    Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
+    if(refTreeSelection)
+      refTreeSelection->select(iter);
+
+    m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
+
+    m_modified = true;
+  }
+
+  enable_buttons();
+}
+
 void Dialog_Layout_Details::on_button_add_related()
 {
   Gtk::TreeModel::iterator parent = get_selected_group_parent();
@@ -789,7 +823,9 @@ void Dialog_Layout_Details::on_button_field_add_group()
   if(iterNewGroup)
   {
     Gtk::TreeModel::Row row = *iterNewGroup;
-    row[m_model_items->m_columns.m_col_layout_item] = sharedptr<LayoutGroup>::create();
+    sharedptr<LayoutGroup> layout_item = sharedptr<LayoutGroup>::create();
+    layout_item->set_name(_("group"));
+    row[m_model_items->m_columns.m_col_layout_item] = layout_item;
 
     //Scroll to, and select, the new row:
     Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
@@ -817,45 +853,17 @@ void Dialog_Layout_Details::on_button_field_formatting()
     {
       Gtk::TreeModel::Row row = *iter;
 
-      try
+      sharedptr<LayoutItem> layout_item = row[m_model_items->m_columns.m_col_layout_item];
+      sharedptr<LayoutItem_Field> field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
+      if(field)
       {
-        Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_layout_field_properties");
-
-        Dialog_FieldLayout* dialog = 0;
-        refXml->get_widget_derived("dialog_layout_field_properties", dialog);
-
-        if(dialog)
+        sharedptr<LayoutItem_Field> chosenitem = offer_field_formatting(field, m_table_name, this);
+        if(chosenitem)
         {
-          add_view(dialog); //Give it access to the document.
-
-          sharedptr<LayoutItem> layout_item = row[m_model_items->m_columns.m_col_layout_item];
-          sharedptr<LayoutItem_Field> field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
-          if(field)
-          {
-            dialog->set_field(field, m_table_name);
-            dialog->set_transient_for(*this);
-            const int response = dialog->run();
-            dialog->hide();
-            if(response == Gtk::RESPONSE_OK)
-            {
-              //Get the chosen field:
-              sharedptr<LayoutItem_Field> itemchosen = dialog->get_field_chosen();
-              if(field)
-              {
-                row[m_model_items->m_columns.m_col_layout_item] = itemchosen;
-                m_modified = true;
-              }
-            }
-          }
-
-          remove_view(dialog);
-          delete dialog;
+          *field = *chosenitem; //TODO_Performance.
+          //m_model_parts->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
         }
       }
-      catch(const Gnome::Glade::XmlError& ex)
-      {
-        std::cerr << ex.what() << std::endl;
-      }
     }
   }
 }
@@ -891,38 +899,69 @@ void Dialog_Layout_Details::on_button_edit()
       }
       else
       {
-        sharedptr<LayoutGroup> layout_group = sharedptr<LayoutGroup>::cast_dynamic(layout_item);
-        if(layout_group)
+        sharedptr<LayoutItem_Notebook> layout_notebook = sharedptr<LayoutItem_Notebook>::cast_dynamic(layout_item);
+        if(layout_notebook)
         {
-            Gtk::TreeModel::Path path = m_model_items->get_path(iter);
-            m_treeview_fields->set_cursor(path, *m_treeview_column_title, true /* start_editing */);
+          Frame_Glom::show_ok_dialog(_("Notebook Tabs"), _("Add child groups to the notebook to add tabs."), *this);
+          /*
+          sharedptr<LayoutItem_Notebook> chosen = offer_notebook(layout_notebook);
+          if(chosen)
+          {
+            row[m_model_items->m_columns.m_col_layout_item] = chosen;
+            m_modified = true;
+          }
+          */
         }
         else
         {
-          sharedptr<LayoutItem_Field> layout_item_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
-          if(layout_item_field)
+          sharedptr<LayoutGroup> layout_group = sharedptr<LayoutGroup>::cast_dynamic(layout_item);
+          if(layout_group)
           {
-            sharedptr<LayoutItem_Field> chosenitem = offer_field_list(layout_item_field);
-            if(chosenitem)
-            {
-              row[m_model_items->m_columns.m_col_layout_item] = chosenitem;
-              m_modified = true;
-            }
+              Gtk::TreeModel::Path path = m_model_items->get_path(iter);
+              m_treeview_fields->set_cursor(path, *m_treeview_column_title, true /* start_editing */);
           }
           else
           {
-            sharedptr<LayoutItem_Button> layout_item_button = sharedptr<LayoutItem_Button>::cast_dynamic(layout_item);
-            if(layout_item_button)
+            sharedptr<LayoutItem_Field> layout_item_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
+            if(layout_item_field)
             {
-              sharedptr<LayoutItem_Button> chosen = offer_button_script_edit(layout_item_button);
-              if(chosen)
+              sharedptr<LayoutItem_Field> chosenitem = offer_field_list(layout_item_field, m_table_name, this);
+              if(chosenitem)
               {
-                *layout_item_button = *chosen;
-                //std::cout << "script: " << layout_item_button->get_script() << std::endl;
-
+                row[m_model_items->m_columns.m_col_layout_item] = chosenitem;
                 m_modified = true;
               }
             }
+            else
+            {
+              sharedptr<LayoutItem_Button> layout_item_button = sharedptr<LayoutItem_Button>::cast_dynamic(layout_item);
+              if(layout_item_button)
+              {
+                sharedptr<LayoutItem_Button> chosen = offer_button_script_edit(layout_item_button);
+                if(chosen)
+                {
+                  *layout_item_button = *chosen;
+                  //std::cout << "script: " << layout_item_button->get_script() << std::endl;
+  
+                  m_modified = true;
+                }
+              }
+              else
+              {
+                sharedptr<LayoutItem_Text> layout_item_text = sharedptr<LayoutItem_Text>::cast_dynamic(layout_item);
+                if(layout_item_text)
+                {
+                  sharedptr<LayoutItem_Text> chosen = offer_textobject(layout_item_text);
+                  if(chosen)
+                  {
+                    *layout_item_text = *chosen;
+                    //std::cout << "script: " << layout_item_button->get_script() << std::endl;
+  
+                    m_modified = true;
+                  }
+                }
+              }
+            }
           }
         }
       }
@@ -1023,14 +1062,7 @@ void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const
           sharedptr<LayoutItem_Field> layout_item_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
           if(layout_item_field)
           {
-            markup = _("Field: "); //TODO: Put this all in a sprintf-style string so it can be translated properly.
-            //Indicate if it's a field in another table.
-            sharedptr<Relationship> relationship = layout_item_field->get_relationship();
-            const Glib::ustring relationship_name = glom_get_sharedptr_name(relationship);
-            if(!(relationship_name.empty()))
-              markup += (relationship_name + "::");
-
-            markup += layout_item_field->get_name();
+            markup = _("Field: ") + layout_item_field->get_layout_display_name(); //TODO: Put this all in a sprintf-style string so it can be translated properly.
 
             //Just for debugging:
             //if(!row[m_model_items->m_columns.m_col_editable])
@@ -1043,8 +1075,16 @@ void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const
             {
               markup = _("Button"); //Buttons don't have names - just titles. TODO: Would they be useful?
             }
-            else if(layout_item)
-              markup = layout_item->get_name();
+            else
+            {
+              sharedptr<LayoutItem_Text> layout_item_text = sharedptr<LayoutItem_Text>::cast_dynamic(layout_item);
+              if(layout_item_text)
+              {
+                markup = _("Text"); //Text objects don't have names - just titles. TODO: Would they be useful?
+              }
+              else if(layout_item)
+                markup = layout_item->get_name();
+            }
           }
         }
       }
@@ -1069,15 +1109,19 @@ void Dialog_Layout_Details::on_cell_data_title(Gtk::CellRenderer* renderer, cons
     {
       Gtk::TreeModel::Row row = *iter;
       sharedptr<LayoutItem> layout_item = row[m_model_items->m_columns.m_col_layout_item];
+      sharedptr<LayoutItem_Notebook> layout_notebook = sharedptr<LayoutItem_Notebook>::cast_dynamic(layout_item);
       if(layout_item)
         renderer_text->property_text() = layout_item->get_title();
+      else if(layout_notebook)
+        renderer_text->property_text() = _("(Notebook)");
       else
         renderer_text->property_text() = Glib::ustring();
 
       sharedptr<LayoutGroup> layout_group = sharedptr<LayoutGroup>::cast_dynamic(layout_item);
       sharedptr<LayoutItem_Portal> layout_portal = sharedptr<LayoutItem_Portal>::cast_dynamic(layout_item);
       sharedptr<LayoutItem_Button> layout_button = sharedptr<LayoutItem_Button>::cast_dynamic(layout_item);
-      const bool editable = (layout_group && !layout_portal) || layout_button; //Only groups and buttons have titles that can be edited.
+      sharedptr<LayoutItem_Text> layout_text = sharedptr<LayoutItem_Text>::cast_dynamic(layout_item);
+      const bool editable = (layout_group && !layout_portal) || layout_button || layout_text; //Only groups, buttons, and text objects have titles that can be edited.
       renderer_text->property_editable() = editable;
     }
   }
