@@ -25,6 +25,7 @@
 #include <gtkmm/treepath.h>
 #include "../../data_structure/layout/layoutitem_field.h"
 #include "../../connectionpool.h"
+#include "../../base_db.h"
 
 class DbTreeModel;
 
@@ -33,8 +34,6 @@ class DbTreeModelRow
 public:
   DbTreeModelRow();
 
-  bool m_placeholder;
-
   typedef Gnome::Gda::Value DbValue;
 
   //Field Values. We store them here after reading them from the database,
@@ -55,6 +54,7 @@ public:
   DbValue m_key;
 
   bool m_removed; //If it should not be shown anymore.
+  bool m_extra; //A temporary new row.
 };
 
 class DbTreeModel
@@ -63,7 +63,9 @@ class DbTreeModel
 {
 public:
   typedef unsigned int size_type;
-  typedef std::vector<LayoutItem_Field> type_vec_fields;
+
+  //typedef std::vector<LayoutItem_Field> type_vec_fields;
+  typedef Base_DB::type_vecLayoutFields type_vec_fields;
 
   friend class DbTreeModelRow;
 
@@ -75,19 +77,24 @@ protected:
 public:
   static Glib::RefPtr<DbTreeModel> create(const Gtk::TreeModelColumnRecord& columns, const Glib::ustring& table_name, const type_vec_fields& column_fields, int column_index_key, bool get_records = true, const Glib::ustring& where_clause = Glib::ustring());
 
-  virtual bool refresh_from_database(const Glib::ustring& where_clause = Glib::ustring());
+  bool refresh_from_database(const Glib::ustring& where_clause = Glib::ustring());
 
   typedef DbTreeModelRow::DbValue DbValue;
 
-  virtual void set_is_placeholder(const TreeModel::iterator& iter, bool val);
-  virtual bool get_is_placeholder(const TreeModel::iterator& iter) const;
+  void set_is_not_placeholder(const TreeModel::iterator& iter);
+  bool get_is_placeholder(const TreeModel::iterator& iter) const;
 
-  virtual void set_key_value(const TreeModel::iterator& iter, const DbValue& value);
-  virtual DbValue get_key_value(const TreeModel::iterator& iter) const;
+  void set_key_value(const TreeModel::iterator& iter, const DbValue& value);
+  DbValue get_key_value(const TreeModel::iterator& iter) const;
 
-  //Convenience method:
+  /** Get the last row - usually the placeholder.
+   */
   TreeModel::iterator get_last_row();
 
+  /** Get the placeholder row.
+   */
+  TreeModel::iterator get_placeholder_row();
+
   /** Removes the given row from the list store.
    * @param iter The iterator to the row to be removed.
    * @result An iterator to the next row, or end() if there is none.
@@ -103,6 +110,8 @@ public:
    */
   iterator append();
 
+  void get_record_counts(gulong& total, gulong& found) const;
+
 protected:
 
    // Overrides:
